#!/bin/bash
#
# Code coverage generation

COVERAGE_DIR="${1}"
REPORT_DIR="${2}"
TYPE="${3}"

cd "$COVERAGE_DIR" || exit 1
PKG_LIST=$(go list ./... | grep -v /vendor/)

# Create the coverage files directory
mkdir -p "$REPORT_DIR";

# Create a coverage file for each package
for package in ${PKG_LIST}; do
    go test -covermode=count -coverprofile "${REPORT_DIR}/${package##*/}.cov" "$package" ;
done ;

# Merge the coverage profile files
echo 'mode: count' > "${REPORT_DIR}"/coverage.cov ;
tail -q -n +2 "${REPORT_DIR}"/*.cov >> "${REPORT_DIR}"/coverage.cov ;

# Display the global code coverage
go tool cover -func="${REPORT_DIR}"/coverage.cov ;

# If needed, generate HTML report
if [ "$TYPE" == "html" ]; then
    go tool cover -html="${REPORT_DIR}"/coverage.cov -o "${REPORT_DIR}/"coverage.html ;
fi

rm -rf ${REPORT_DIR}"/"*.cov

# Remove the coverage files directory
cd - || exit 1
