# See full reference at https://devenv.sh/reference/options/
{ pkgs, inputs, phps, lib, config, modulesPath,... }:

{

  env.LD_LIBRARY_PATH = lib.makeLibraryPath [  ];

  # https://devenv.sh/packages/
  packages = [
    inputs.version.defaultPackage."${builtins.currentSystem}"
    pkgs.git
    pkgs.gcc12
    #pkgs.glibc
    pkgs.go-task
    pkgs.blackbox
    pkgs.blackbox-terminal
    pkgs.jq
    pkgs.delve
    pkgs.gdlv
    pkgs.wget
    pkgs.unixtools.xxd
    pkgs.libffi
    pkgs.zlib
    pkgs.procps
    pkgs.php81Extensions.xdebug
    pkgs.ranger
    pkgs.meld
    pkgs.gnused
    pkgs.coreutils-full # # https://search.nixos.org/packages?channel=23.05&show=coreutils-full&from=0&size=50&sort=relevance&type=packages&query=mkdir
    pkgs.gnugrep
    pkgs.gnumake
    pkgs.util-linux
    #    pkgs.logrotate
    pkgs.httpie
    pkgs.netcat
    pkgs.memcached
    pkgs.fd
    
    #pkgs.process-compose
    #    pkgs.glibc_multi
    #    pkgs.curl
  ];

  ## curl geht nicht!

  # https://devenv.sh/languages/
  # languages.nix.enable = true;
  languages = {
    go = { enable = true; };
  };
  
   difftastic.enable = true;

    
  # This script is executed when the app is built
  # You can use it to build the app  
  scripts.build-app.exec =  ''
#!${pkgs.bash}/bin/bash
#set -euo pipefail
set -x

commands_coreutils=("echo" "cut" "paste" "md5sum" "wc" "pwd" "sort" "realpath" "rm" "cat" "touch" "stat" "mkdir" "ln" "trap" "env")

for cmd in "''${commands_coreutils[@]}"; do
    eval "''${cmd}() { ${pkgs.coreutils}/bin/''${cmd} \"\$@\"; }"
    export -f "''${cmd}"
done

find() {
  ${pkgs.findutils}/bin/find "$@"
}

export -f find

rsync() {
  ${pkgs.rsync}/bin/rsync "$@"
}

export -f rsync

sh() {
  ${pkgs.bash}/bin/sh "$@"
}

export -f sh

exa() {
  ${pkgs.exa}/bin/exa "$@"
}

export -f exa

grep() {
  ${pkgs.gnugrep}/bin/grep "$@"
}

export -f grep

ps() {
  ${pkgs.procps}/bin/ps "$@"
}

export -f ps

pgrep() {
  ${pkgs.procps}/bin/pgrep "$@"
}

export -f pgrep

set -x

version=''$(git describe --tags --always --dirty)
buildDate=''$(date -u +"%Y-%m-%dT%H:%M:%SZ")


${config.devenv.root}/bin/go-compile.sh \
                    "${config.devenv.root}/source/" \
                    "${config.devenv.root}/dist"/ \
                    "bob" \
                    "main" \
                    "''${version}" \
                    "''${buildDate}"


'';



  # This scritp is used to deploy the app to the gitlab registry
  # It is used by the gitlab-ci.yml file
  # The environment variables are set in the gitlab project settings
  scripts.deploy-app.exec =  ''
#!${pkgs.bash}/bin/bash

commands_coreutils=("echo" "mv" "cut" "sleep" "md5sum"  "pwd" "sort" "realpath" "rm" "cat" "touch" "stat" "mkdir" "ln" "trap" "env")

for cmd in "''${commands_coreutils[@]}"; do
    eval "''${cmd}() { ${pkgs.coreutils}/bin/''${cmd} \"\$@\"; }"
    export -f "''${cmd}"
done

jq() {
  ${pkgs.jq}/bin/jq "$@"
}

export -f jq

curl() {
  ${pkgs.curl}/bin/curl "$@"
}

export -f curl

sponge() {
  ${pkgs.moreutils}/bin/sponge "$@"
}

export -f sponge

tar() {
  ${pkgs.gnutar}/bin/tar "$@"
}

export -f tar

gzip() {
  ${pkgs.gzip}/bin/gzip "$@"
}

export -f gzip

exa() {
  ${pkgs.exa}/bin/exa "$@"
}

export -f exa

git() {
  ${pkgs.git}/bin/git "$@"
}

export -f git

version() {
  ${inputs.version.defaultPackage."${builtins.currentSystem}"}/bin/version "$@"
}

export -f version

if [[ -f .env-gitlab-ci ]]; then
    source .env-gitlab-ci
    rm .env-gitlab-ci
fi

set -x
## if $HOME not set, set it to current directory
if [[ -z "''${HOME}" ]]; then
    HOME=$(pwd)
fi

export HOME

git config user.email "''${GITLAB_USER_EMAIL}"
git config user.name "''${GITLAB_USER_NAME:-"Gitlab CI"}"
git config pull.rebase true
git config http.sslVerify "false"
git remote set-url origin https://pad:''${GITLAB_TOKEN}@''${CI_REPOSITORY_URL#*@}

git fetch --all --tags --unshallow
git reset --hard origin/master
git checkout $CI_COMMIT_REF_NAME
git pull origin $CI_COMMIT_REF_NAME


if [ ! -z "''${CI_PROJECT_DIR}" ]; then
    echo "CI_PROJECT_DIR is set, using it as project root."
    project_root=$(realpath "''${CI_PROJECT_DIR}")/
elif [ ! -z "''${DEVENV_ROOT}" ]; then
    echo "DEVENV_ROOT is set, using it as project root."
    project_root=$(realpath "''${DEVENV_ROOT}")/
else
    echo "Error: DEVENV_ROOT or CI_PROJECT_DIR environment variables are not set."
    exit 1  
fi

if [ ! -d "''${project_root}" ]; then
    echo "Error: Project root directory does not seem to be valid."
    echo "Check the DEVENV_ROOT or CI_PROJECT_DIR environment variables."
    exit 1
fi

if [ -z "'CI_JOB_TOKEN" ]; then
    echo "Error: CI_JOB_TOKEN variable is not set."
    exit 1
fi

git --no-pager log --decorate=short --pretty=oneline

if ! version auto --git --verbose
then
    echo "ERROR: Could not update version."
    exit 1
fi 

git --no-pager log --decorate=short --pretty=oneline

gitVersion=$(version print --git)
packageJsonFile=''${project_root}package.json

app_name=$(jq -r '.name' ''${packageJsonFile})

jq ".version = \"''${gitVersion}\"" ''${packageJsonFile} | sponge ''${packageJsonFile}
 
archiveFile="''${project_root}''${app_name}-''${gitVersion}.tar.gz"

tar -C ''${project_root}dist -czf ''${archiveFile} . 

curl -v --insecure -H "JOB-TOKEN: ''${CI_JOB_TOKEN}" --upload-file \
    ''${archiveFile} \
    https://gitlab.schukai.com/api/v4/projects/''${CI_PROJECT_ID}/packages/generic/''${app_name}/''${gitVersion}/''${app_name}.tar.gz  

rm ''${archiveFile}

# Check if the file has changes
if [[ -n $(git status --porcelain "''${packageJsonFile}") ]]; then
    echo "Changes found in ''${packageJsonFile}. Committing..."
    git add "''${packageJsonFile}"
    git commit -m "Bump version to ''${gitVersion}"
    git push -o ci.skip origin ''${CI_COMMIT_REF_NAME} --tags
else
    echo "No changes found in ''${CI_COMMIT_REF_NAME}. Skipping commit."
fi

echo "done"

  '';


}
